@@ -5,8 +5,8 @@
 ;; Author: Earl Hyatt
 ;; Created: November 2020
 ;; URL: https://github.com/okamsn/loopy
-;; Package-Version: 20251116.13
-;; Package-Revision: 07a9952cd6c0
+;; Package-Version: 20260105.316
+;; Package-Revision: 2173cd58e3a5
 ;; Package-Requires: ((emacs "28.1") (map "3.3.1") (seq "2.22") (compat "29.1.3") (stream "2.4.0"))
 ;; Keywords: extensions
 ;; LocalWords:  Loopy's emacs Edebug
@@ -157,69 +157,34 @@ this means that an explicit \"nil\" is always required."
     (error "Invalid binding in `loopy' expansion: %s" binding)))
 
 (defun loopy--destructure-for-with-vars (bindings)
-  "Destructure BINDINGS into bindings suitable for something like `let*'.
+  "Get function to wrap code and destructure values in BINDINGS.
 
 This function named by this variables receives the bindings given
 to the `with' macro argument and should usually return a list of
 two elements:
 
-1. A function/macro that works like `let*' and can be used to wrap
-   the expanded macro code.
-2. The bindings that will be given to this macro.
-
-For example, an acceptable return value might be something like
-
-    (list \\='pcase-let* BINDINGS)
-
-which will be used to wrap the loop and other code."
+1. A list of symbols being all the variables to be bound in BINDINGS.
+2. A function to be called with the code to be wrapped, which
+  should produce wrapped code appropriate for BINDINGS,
+  such as a `let*' form."
   (funcall (or loopy--destructuring-for-with-vars-function
                #'loopy--destructure-for-with-vars-default)
            bindings))
 
 (defun loopy--destructure-for-with-vars-default (bindings)
-  "Destructure BINDINGS into bindings suitable for something like `let*'.
+  "Get function to wrap code and destructure values in BINDINGS.
 
 Returns a list of two elements:
-1. The symbol `pcase-let*'.
-2. A new list of bindings."
-  ;; We do this instead of passing to `pcase-let*' so that:
-  ;; 1) We sure that variables are bound even when unmatched.
-  ;; 2) We can signal an error if the pattern doesn't match a value.
-  ;; This keeps the behavior of the old implementation.
-  ;;
-  ;; Note: Binding the found variables to `nil' would overwrite any values that
-  ;;       we might try to access while binding, so we can't do that like we do
-  ;;       for iteration commands in which we already know the scope.
-  ;; (let ((new-binds)
-  ;;       (all-set-exprs))
-  ;;   (dolist (bind bindings)
-  ;;     (cl-destructuring-bind (var val)
-  ;;         bind
-  ;;       (if (symbolp var)
-  ;;           (push `(,var ,val) new-binds)
-  ;;         (let ((sym (gensym)))
-  ;;           (push `(,sym ,val) new-binds)
-  ;;           (cl-destructuring-bind (set-expr found-vars)
-  ;;               (loopy--pcase-destructure-for-iteration `(loopy ,var) sym :error t)
-  ;;             (dolist (v found-vars)
-  ;;               (push `(,v nil) new-binds))
-  ;;             (push set-expr all-set-exprs))))))
-  ;;   (list 'let* (nreverse new-binds) (macroexp-progn (nreverse
-  ;;                                                     all-set-exprs))))
-  (let ((new-binds))
-    (dolist (bind bindings)
-      (cl-destructuring-bind (var val)
-          bind
-        (if (symbolp var)
-            (push `(,var ,val) new-binds)
-          (let ((sym (gensym)))
-            (push `(,sym ,val) new-binds)
-            (cl-destructuring-bind (set-expr found-vars)
-                (loopy--pcase-destructure-for-iteration `(loopy ,var) sym :error t)
-              (dolist (v found-vars)
-                (push `(,v nil) new-binds))
-              (push `(_ ,set-expr) new-binds))))))
-    (list 'let* (nreverse new-binds))))
+1. A list of symbols being all the variables to be bound in BINDINGS.
+2. A function to be called with the code to be wrapped, which
+  should produce wrapped code appropriate for BINDINGS,
+  such as a `let*' form."
+  (loopy--pcase-destructure-for-with-vars (cl-loop for b in bindings
+                                                   for (var val) = b
+                                                   collect (if (symbolp var)
+                                                               b
+                                                             `((loopy ,var) ,val)))
+                                          :error t))
 
 ;;;; The Macro Itself
 (defun loopy--expand-to-loop ()
@@ -427,8 +392,7 @@ The function creates quoted code that should be used by a macro."
 
       ;; Declare the With variables.
       (when loopy--with-vars
-        (setq result `(,@(loopy--destructure-for-with-vars loopy--with-vars)
-                       ,@(get-result))
+        (setq result (funcall (cl-second loopy--with-vars) (get-result))
               result-is-one-expression t))
 
       ;; Declare the symbol macros.
@@ -528,14 +492,13 @@ Returns BODY without the `%s' argument."
 
 (loopy--def-special-processor with
   (setq loopy--with-vars
-        ;; Note: These values don't have to be used literally, due to
-        ;;       destructuring.
-        (mapcar (lambda (binding)
-                  (cond ((symbolp binding)      (list binding nil))
-                        ((= 1 (length binding)) (list (cl-first binding)
-                                                      nil))
-                        (t                       binding)))
-                arg-value))
+        (loopy--destructure-for-with-vars
+         (mapcar (lambda (binding)
+                   (cond ((symbolp binding)      (list binding nil))
+                         ((= 1 (length binding)) (list (cl-first binding)
+                                                       nil))
+                         (t                       binding)))
+                 arg-value)))
   (seq-remove (lambda (x) (eq (car x) arg-name)) body))
 
 (loopy--def-special-processor without
@@ -659,21 +622,14 @@ macro `loopy' itself."
        ;; Don't want to accidentally rebind variables to `nil'
        ;; or to accidentally mis-use commands that need
        ;; different initial values.
-       (loopy--pcase-let-workaround (var new-val)
-         (pcase-let ((`(,var ,new-val) instruction-value))
-           (pcase var
-             ((pred loopy--with-bound-p) nil)
-             ((and (app loopy--command-bound-p `(,_place . ,old-val))
-                   (guard (not (equal new-val old-val))))
-              ;; TODO: Switch from raising a warning to raising an error.
-              ;; (signal 'loopy-incompatible-accumulation-initializations
-              ;;         (list :in place :var var :old old-val :new new-val))
-              (display-warning
-               'loopy
-               (format "loopy: Conflicting accumulation starting values: `%s', %s, %s\nThis will be an error in the future.  To resolve this error, use `with' to explicitly specify a starting value."
-                       var old-val new-val)
-               :warning))
-             (_ (push instruction-value loopy--accumulation-vars))))))
+       (pcase-let ((`(,var ,new-val) instruction-value))
+         (pcase var
+           ((pred loopy--with-bound-p) nil)
+           ((and (app loopy--command-bound-p `(,place . ,old-val))
+                 (guard (not (equal new-val old-val))))
+            (signal 'loopy-incompatible-accumulation-initializations
+                    (list :in place :var var :old old-val :new new-val)))
+           (_ (push instruction-value loopy--accumulation-vars)))))
 
       (loopy--other-vars
        (loopy--validate-binding instruction-value)
@@ -1044,7 +1000,8 @@ instead of this macro.
   (declare (debug (&rest [sexp form])))
   (macroexp-progn
    (cl-loop for (var val) on args by #'cddr
-            collect (car (loopy--destructure-for-iteration-default var val)))))
+            collect (car (loopy--pcase-destructure-for-iteration
+                          `(loopy ,var) val :error t)))))
 
 ;;;###autoload
 (defmacro loopy-let* (bindings &rest body)
@@ -1056,25 +1013,25 @@ you wish to use `pcase' destructuring, you should use `pcase-let'
 instead of this macro."
   (declare (debug ((&rest [sexp form]) body))
            (indent 1))
-  ;; Because Emacs versions less than 28 weren't guaranteed to bind all
-  ;; variables in Pcase, we need to use the same approach we do for
-  ;; destructuring `with' bindings, instead of just passing the bindings to
-  ;; `pcase' directly.
-  (let ((new-binds))
-    (dolist (bind bindings)
-      (cl-destructuring-bind (var val)
-          bind
-        (if (symbolp var)
-            (push bind new-binds)
-          (let ((sym (gensym)))
-            (push `(,sym ,val) new-binds)
-            (cl-destructuring-bind (var-set-expr var-list)
-                (loopy--pcase-destructure-for-iteration `(loopy ,var) sym :error t)
-              (dolist (var var-list)
-                (push var new-binds))
-              (push `(_ ,var-set-expr) new-binds))))))
-    `(let* ,(nreverse new-binds)
-       ,@body)))
+  ;; NOTE: We don't use `pcase-let*' here because we want to keep
+  ;;       the signal of `loopy-bad-run-time-destructuring'.
+  (cl-flet ((pcase-maker ((var val) body)
+              ;; Using `(loopy SYMBOL)' as a pattern still triggers
+              ;; the warning for shadowing the all-matching pattern `fail',
+              ;; so instead we use a `let' binding when VAR is a symbol
+              ;; to avoid `pcase'.
+              (if (symbolp var)
+                  `(let ((,var ,val))
+                     ,body)
+                `(pcase ,val
+                   ((loopy ,var) ,body)
+                   (fail (signal 'loopy-bad-run-time-destructuring
+                                 (list (quote (loopy ,var)) fail)))))))
+    (cl-loop with rev = (reverse bindings)
+             with result = (pcase-maker (car rev) (macroexp-progn body))
+             for bind in (cdr rev)
+             do (setq result (pcase-maker bind result))
+             finally return result)))
 
 ;;;###autoload
 (defmacro loopy-ref (bindings &rest body)
